-- PARTIE 2 : Création et modification du schema / insertion
-- Julien Maître
-- Paul Narbonneau


-- Q 2
CREATE TABLE CLI (
    NumCli INT NOT NULL PRIMARY KEY ,
    NomCli VARCHAR(255) NOT NULL ,
    Pays VARCHAR(255) NOT NULL
);

CREATE TABLE PROD (
    NumProd INT NOT NULL PRIMARY KEY ,
    NomProd VARCHAR(255) NOT NULL ,
    TypeProd VARCHAR(255) NOT NULL
);

CREATE TABLE COM (
    NumCom INT NOT NULL PRIMARY KEY ,
    NumCli INT NOT NULL REFERENCES CLI(NumCli) ,
    FraisPort FLOAT NOT NULL ,
    AnCom INT NOT NULL
);

CREATE TABLE DET (
    NumCom INT NOT NULL,
    NumProd INT NOT NULL,
    Qte INT NOT NULL,
    Remise FLOAT NOT NULL,
    PRIMARY KEY (NumCom, NumProd),
    CONSTRAINT FK_DET_TO_COM FOREIGN KEY (NumCom) REFERENCES COM(NumCom),
    CONSTRAINT FK_DET_TO_PROD FOREIGN KEY (NumProd) REFERENCES PROD(NumProd)
);

-- Q3
-- Oui, création obligatoire des objets avant leurs référencement.

-- Q5, Q6, Q7, Q8 :
ALTER TABLE CLI MODIFY (NomCli char(30));
ALTER TABLE CLI ADD (Tel VARCHAR(20));
ALTER TABLE PROD ADD (PrixUnit FLOAT);
-- avant Q8
ALTER TABLE PROD MODIFY (NomProd NULL);
ALTER TABLE PROD MODIFY (NomProd NOT NULL);

-- Q9 :
-- Génération de Clients
CREATE OR REPLACE PROCEDURE make_CLI IS
BEGIN
    For i in 1..100
    Loop
        CASE MOD(i, 3)
            WHEN 0 THEN INSERT INTO CLI (NumCli, NomCli, Pays, Tel) VALUES (i, to_char('Client_' || i), 'France', NULL) ;
            WHEN 1 THEN INSERT INTO CLI (NumCli, NomCli, Pays, Tel) VALUES (i, to_char('Client_' || i), 'Allemagne', NULL) ;
            WHEN 2 THEN INSERT INTO CLI (NumCli, NomCli, Pays, Tel) VALUES (i, to_char('Client_' || i), 'Belgique', NULL) ;
        End Case;
    End loop;
End;
/

-- Génération de Produits
CREATE OR REPLACE PROCEDURE make_PROD IS
BEGIN
    For i in 1..100
    Loop
        INSERT INTO PROD (NumProd, NomProd, TypeProd, PrixUnit) VALUES (i, to_char('Produit_' || i), to_char('TypeProd_' || i), MOD(i*13, 25)+1) ;
    End loop;
End;
/

-- Génération de Commandes
CREATE OR REPLACE PROCEDURE make_COM IS
BEGIN
    For i in 1..100
    Loop
        INSERT INTO COM (NumCom, NumCli, FraisPort, AnCom) VALUES (i, Mod(i*7, 100)+1, MOD(i*3, 5), 1990+MOD(i, 20)) ;
    End loop;
End;
/

-- Génération de DET
CREATE OR REPLACE PROCEDURE make_DET IS
BEGIN
    For i in 1..100
    Loop
        INSERT INTO DET (NumCom, NumProd, Qte, Remise) VALUES (i, Mod(i*11, 100)+1, MOD(i, 3)*3+1, MOD(i+5, 10)) ;
    End loop;
End;
/

-- Génération complète
CREATE OR REPLACE PROCEDURE make_all IS
BEGIN
    delete from det;
    delete from com;
    delete from prod;
    delete from cli;

    make_cli();
    make_prod();
    make_com();
    make_det();
End;
/

-- Q 11
UPDATE CLI SET NomCli = UPPER(NomCli);

-- Q 12
UPDATE DET
SET Remise = Remise * 2
WHERE NumCom IN (
    SELECT NumCom
    FROM COM
    WHERE AnCom > 1996
);

-- Q 13

DELETE FROM DET WHERE NumCom IN (
    SELECT NumCom
    FROM COM
    WHERE NumCli IN (
        SELECT NumCli
        FROM CLI
        WHERE Pays LIKE 'France'
    )
);

DELETE FROM COM WHERE NumCli IN (
    SELECT NumCli
    FROM CLI
    WHERE Pays LIKE 'France'
);

DELETE FROM CLI WHERE Pays LIKE 'France';

-- Q 14
-- Selection des utilisateurs ...
SELECT *
FROM CLI,
    COM
WHERE
    CLI.NumCli = COM.NumCli
AND
    COM.AnCom < 1991;


-----------------------------------------------------------------------
-- ça commence ici
-----------------------------------------------------------------------
DELETE FROM DET WHERE NumCom IN (
    SELECT NumCom
    FROM COM
    WHERE NumCli IN (
        SELECT CLI.NumCli
        FROM CLI,
            COM
        WHERE
            CLI.NumCli = COM.NumCli
        AND
            COM.AnCom < 1991
    )
);

CREATE TABLE CLI_DEL_TEMP (
    NumCli INT PRIMARY KEY
);

INSERT INTO CLI_DEL_TEMP (NumCli)
SELECT CLI.NumCli
FROM CLI,
    COM
WHERE
    CLI.NumCli = COM.NumCli
AND
    COM.AnCom < 1991;

DELETE FROM COM WHERE NumCli IN (
    SELECT CLI.NumCli
    FROM CLI,
        COM
    WHERE
        CLI.NumCli = COM.NumCli
    AND
        COM.AnCom < 1991
);

DELETE FROM CLI WHERE NumCli IN (
    SELECT NumCli
    FROM CLI_DEL_TEMP
);

DROP TABLE CLI_DEL_TEMP;

-- Q 15
-- Ordre inverse de leur création, contrainte de clé étrangère : on supprime les références avant les tuples eux mêmes
delete from det;
delete from com;
delete from prod;
delete from cli;

-- Q 16
-- Ordre inverse de leur création, contrainte de clé étrangère : on supprime les références avant les tuples eux mêmes
drop table det;
drop table com;
drop table prod;
drop table cli;
