-- PARTIE 5 : Experimentations sur l'optimisation
-- Julien Maître
-- Paul Narbonneau

-- Q1
/*
On récupère les timestamps avant insertion et après insertion :
on obtient le temps écoulé pour insérer les tuples

CLEAN -> DET
avec index : 2,252 / 2,035 / 2,081 / 3,417 / 2,583 / 4,095 / 4,215 / 3,982 / 4,268
sans index : 2,391 / 2,619 / 1,869 / 1,901 / 2,224 / 3,898 / 4,502 / 3,924 / 5,395

CLEAN -> *
avec index : 6,066 / 6,112 / 5,784 / 6,201
sans index : 5,709 / 7,528 / 5,749 / 7,150
*/

Drop TABLE TEMPS;

CREATE TABLE TEMPS
( time_id int(3), -- id pour plusieurs mesures simultanées
date_debut TIMESTAMP(3),
date_fin TIMESTAMP(3));

-- Première mesure
insert into TEMPS values (1,SYSTIMESTAMP,null);

/* Insertions SQL du fichier CLEAN.sql */

-- Mise à jour des dates de fin
update TEMPS set date_fin=SYSTIMESTAMP;

-- Affichage
select time_id,(date_fin - date_debut) from temps;

-- Q2
-- R1
SELECT
    count(distinct L.NumCli)
FROM
    CLI L,
    COM C,
    DET D,
    PRO P
WHERE
    L.NumCli = C.NumCli
AND
    C.NumCom = D.NumCom
AND
    D.NumPro = P.NumPro;

/* Résultat :
avec index :
SELECT_STATEMENT(
    SORT_GROUP_BY(                                  -- distinct
        HASH_JOIN(                                  -- C x D
            NESTED_LOOPS(                           -- L x C
                CLI.PK,
                FULL_SCAN(COM)
            ),
            DET.PK
        )
    )
)
Aucun accès à P ?

sans index :
SELECT_STATEMENT(
    SORT_GROUP_BY(                              -- distinct
        HASH_JOIN(                              -- D x P
            TABLE_ACCESS_FULL(PRO),
            HASH_JOIN(                          -- C x D
                HASH_JOIN(                      -- L x C
                    TABLE_ACCESS_FULL(CLI),
                    TABLE_ACCESS_FULL(COM)
                ),
                TABLE_ACCESS_FULL(DET)
            )
        )
    )
)

Sans index les couts sont très largement supérieur en CPU : Scan complet de toutes les tables concernées
Avec index, le SGBD élimine directement certaines tables, et utilise les index pour minimiser les scans complets


*/

-- R2
SELECT
    count(distinct L.CodeCli)
FROM
    CLI L
WHERE
    L.CodeCli IN (
        SELECT
            C.CodeCli
        FROM
            COM C
        WHERE
            C.NumCom IN (
                SELECT
                    D.NumCom
                FROM
                    DET D
                WHERE
                    D.NumPro IN (
                        SELECT
                            P.NumPro
                        FROM
                            PRO P
                    )
            )
    )
;
/* Résultat :

*/


-- R3
SELECT
    count(distinct L.CodeCli)
FROM
    CLI L
WHERE
    EXISTS (/*???*/)






-- R5
SELECT
    count(distinct L.CodeCli)
FROM
    CLI L,
    COM C,
    DET D,
    PRO P
WHERE
    L.CodeCli = C.CodeCli
AND
    C.NumCom = D.NumCom
AND
    D.NumPro = P.NumPro
AND
    NumCom = 10000;
/* Résultat :

*/

-- R6
SELECT
    count(distinct L.CodeCli)
FROM
    CLI L,
    COM C,
    DET D,
    PRO P
WHERE
    L.CodeCli = C.CodeCli
AND
    C.NumCom = D.NumCom
AND
    D.NumPro = P.NumPro
AND
    NumCom > 10000;
/* Résultat :


*/


